﻿// $Header: $ 
// Copyright (c) CODE Consulting and Development, s.r.o., Plzen. All rights reserved.
using System;
using System.Globalization;
using System.Reflection;
using System.Windows.Forms;
using EasyOpcUADemo.Properties;
using JetBrains.Annotations;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.OperationModel;

[assembly:CLSCompliant(true)]
namespace EasyOpcUADemo
{
    public partial class MainForm : Form
    {
        // ReSharper disable once NotNullMemberIsNotInitialized
        public MainForm()
        {
            InitializeComponent();
        }


        private UANodeDescriptor _NodeDescriptor;
        public UANodeDescriptor NodeDescriptor
        {
            get { return this._NodeDescriptor ?? (this._NodeDescriptor = new UANodeDescriptor()); }
            set
            {
                if (value == this._NodeDescriptor) return;
                this._NodeDescriptor = value;
            }
        }

        // ReSharper disable InconsistentNaming
        private void readButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            UAAttributeData attributeData = null;
            Exception exception = null;
            try
            {

                // ReSharper disable AssignNullToNotNullAttribute
                attributeData = easyUAClient1.Read(serverUriTextBox.Text,
                     this.chb_useBrowsePaht.Checked ? this.NodeDescriptor : nodeIdTextBox.Text
                    );
                var ident  = easyUAClient1.Read(
                    serverUriTextBox.Text,
                    this.chb_useBrowsePaht.Checked ? this.NodeDescriptor : nodeIdTextBox.Text,
                    UAAttributeId.NodeId);

                // ReSharper restore AssignNullToNotNullAttribute
            }
            catch (UAException ex)
            {
                exception = ex;
            }
            DisplayAttributeData(attributeData);
            DisplayException(exception);
        }

        private void DisplayAttributeData(UAAttributeData attributeData)
        {
            if (attributeData == null)
            {
                valueTextBox.Text = "";
                statusTextBox.Text = "";
                sourceTimestampTextBox.Text = "";
                serverTimestampTextBox.Text = "";
            }
            else
            {
                valueTextBox.Text = attributeData.DisplayValue();
                statusTextBox.Text = attributeData.StatusCode.ToString();
                sourceTimestampTextBox.Text = attributeData.SourceTimestamp.ToString(CultureInfo.CurrentCulture);
                serverTimestampTextBox.Text = attributeData.ServerTimestamp.ToString(CultureInfo.CurrentCulture);
            }
        }

        private void DisplayException(Exception exception)
        {
            exceptionTextBox.Text = exception == null ? "" : exception.GetBaseException().Message;
        }

        private bool _isSubscribed/* = false*/;
        private int _handle/* = 0*/;

        public bool IsSubscribed
        {
            get { return _isSubscribed; }
            set 
            {
                _isSubscribed = value;
                subscribeMonitoredItemButton.Enabled = !_isSubscribed;
                changeMonitoredItemSubscriptionButton.Enabled = _isSubscribed;
                unsubscribeMonitoredItemButton.Enabled = _isSubscribed;
            }
        }

        // ReSharper disable InconsistentNaming
        private void subscribeButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // ReSharper disable AssignNullToNotNullAttribute

            _handle = easyUAClient1.SubscribeDataChange(
                serverUriTextBox.Text, 
                this.chb_useBrowsePaht.Checked ? this.NodeDescriptor : nodeIdTextBox.Text,
                (int)samplingIntervalNumericUpDown.Value);


            // ReSharper restore AssignNullToNotNullAttribute
            IsSubscribed = true;
        }

        // ReSharper disable InconsistentNaming
        private void changeSubscriptionButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            easyUAClient1.ChangeMonitoredItemSubscription(_handle, (int)samplingIntervalNumericUpDown.Value);
        }

        // ReSharper disable InconsistentNaming
        private void unsubscribeButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            Unsubscribe();
        }

        private void Unsubscribe()
        {
            easyUAClient1.UnsubscribeMonitoredItem(_handle);
            _handle = 0;
            IsSubscribed = false;
        }

        // ReSharper disable InconsistentNaming
        private void easyUAClient1_DataChangeNotification([NotNull] object sender, [NotNull] EasyUADataChangeNotificationEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            DisplayAttributeData((e.Exception == null) ? e.AttributeData : null);
            DisplayException(e.Exception);
        }

        // ReSharper disable InconsistentNaming
        private void aboutButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            MessageBox.Show(this, Assembly.GetExecutingAssembly().FullName, Resources.MainForm_AssemblyName, 
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // ReSharper disable InconsistentNaming
        private void closeButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            if (IsSubscribed)
                Unsubscribe();
            Close();
        }

        // ReSharper disable InconsistentNaming
        private void MainForm_Load(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {

        }

        // ReSharper disable InconsistentNaming
        private void discoverServersButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // ReSharper disable AssignNullToNotNullAttribute
            uaHostAndEndpointDialog1.EndpointDescriptor = serverUriTextBox.Text;
            // ReSharper restore AssignNullToNotNullAttribute
            if (uaHostAndEndpointDialog1.ShowDialog() == DialogResult.OK)
                serverUriTextBox.Text = uaHostAndEndpointDialog1.EndpointDescriptor.UrlString;
        }

        // ReSharper disable InconsistentNaming
        private void browseDataButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            uaDataDialog1.EndpointDescriptor = serverUriTextBox.Text;
            if (uaDataDialog1.ShowDialog() == DialogResult.OK)
                // ReSharper disable PossibleNullReferenceException
                nodeIdTextBox.Text = uaDataDialog1.NodeDescriptor.NodeId;
                tb_browsePath.Text = uaDataDialog1.NodeDescriptor.BrowsePath.ToString();
            this.NodeDescriptor = new UANodeDescriptor(uaDataDialog1.NodeDescriptor.BrowsePath);


            // ReSharper restore PossibleNullReferenceException
        }

        // ReSharper disable InconsistentNaming
        private void writeValueButton_Click(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
            Exception exception = null;
            try
            {
                // ReSharper disable AssignNullToNotNullAttribute
                easyUAClient1.WriteValue(serverUriTextBox.Text, nodeIdTextBox.Text, valueToWriteTextBox.Text);
                // ReSharper restore AssignNullToNotNullAttribute
            }
            catch (UAException ex)
            {
                exception = ex;
            }
            DisplayException(exception);
        }
                
    }
}
